@@ -64,6 +64,7 @@ enum GGUFArch
     ARCH_GLM4 = 9,
     ARCH_GEMMA3N = 10,
     ARCH_JAMBA = 11,
+    ARCH_GPTOSS = 12,
 };
 
 struct FileFormatExtraMeta
@@ -130,6 +131,8 @@ FileFormat check_file_format(const std::string & fname, FileFormatExtraMeta * fi
 void ContextFastForward(std::vector<int> &current_context_tokens, std::vector<int> &embd_inp,
  int &n_past, std::vector<int> &last_n_tokens, const int nctx, std::vector<int> &smartcontext,
  const bool useSmartContext, const bool requireFullSubset);
+bool gguf_tensor_exists(const std::string & filename, std::string tensor_name, bool exactmatch);
+std::string gguf_get_model_arch(const std::string & filename);
 
 size_t gpttype_calc_new_state_kv();
 size_t gpttype_calc_new_state_tokencount();
